@@ -23,6 +23,7 @@ import utils
 # Force certain helpers to return consistent values
 virtinst._util.is_blktap_capable = lambda: False
 virtinst._util.default_bridge2 = lambda ignore1: ["bridge", "eth0"]
+virtinst.Guest._open_uri = lambda ignore1, ignore2: None
 
 # Setup logging
 rootLogger = logging.getLogger()
@@ -50,3 +51,4 @@ import nodedev
 import virtconvtest
 import interface
 import xmlparse
+import support
