@@ -32,6 +32,7 @@ remoteuri   = fakeuri + ",remote"
 kvmuri      = fakeuri + ",caps=`pwd`/tests/capabilities-xml/libvirt-0.7.6-qemu-caps.xml,qemu"
 xenuri      = fakeuri + ",caps=`pwd`/tests/capabilities-xml/rhel5.4-xen-caps-virt-enabled.xml,xen"
 xenia64uri  = fakeuri + ",caps=`pwd`/tests/capabilities-xml/xen-ia64-hvm.xml,xen"
+lxcuri      = fakeuri + ",caps=`pwd`/tests/capabilities-xml/capabilities-lxc.xml,lxc"
 
 # Location
 image_prefix = "/tmp/__virtinst_cli_"
@@ -81,10 +82,12 @@ test_files = {
     'KVMURI'            : kvmuri,
     'XENURI'            : xenuri,
     'XENIA64URI'        : xenia64uri,
+    'LXCURI'            : lxcuri,
     'CLONE_DISK_XML'    : "%s/clone-disk.xml" % xmldir,
     'CLONE_STORAGE_XML' : "%s/clone-disk-managed.xml" % xmldir,
     'CLONE_NOEXIST_XML' : "%s/clone-disk-noexist.xml" % xmldir,
     'IMAGE_XML'         : "%s/image.xml" % xmldir,
+    'IMAGE_NOGFX_XML'   : "%s/image-nogfx.xml" % xmldir,
     'NEWIMG1'           : new_images[0],
     'NEWIMG2'           : new_images[1],
     'NEWIMG3'           : new_images[2],
@@ -206,6 +209,8 @@ args_dict = {
         "--disk %(EXISTIMG1)s,driver_name=foobar,driver_type=foobaz",
         # Using a storage pool source as a disk
         "--disk /dev/hda",
+        # Building 'default' pool
+        "--disk pool=default,size=.00001",
       ],
 
       "invalid": [
@@ -345,6 +350,34 @@ args_dict = {
 
      }, # category "graphics"
 
+     "smartcard": {
+      "args": "--noautoconsole --nodisks --pxe",
+
+      "valid": [
+        # --smartcard host
+        "--smartcard host",
+        # --smartcard none,
+        "--smartcard none",
+        # --smartcard mode with type
+        "--smartcard passthrough,type=spicevmc",
+        # --smartcard mode with type
+        # XXX Requires implementing more opts
+        #"--smartcard passthrough,type=tcp",
+      ],
+
+      "invalid": [
+        # Missing argument
+        "--smartcard",
+        # Invalid argument
+        "--smartcard foo",
+        # Invalid type
+        "--smartcard passthrough,type=foo",
+        # --smartcard bogus
+        "--smartcard host,foobar=baz",
+      ],
+
+     }, # category "smartcard"
+
     "char" : {
      "args": "--hvm --nographics --noautoconsole --nodisks --pxe",
 
@@ -415,6 +448,8 @@ args_dict = {
         "--cpu somemodel",
         # Crazy --cpu
         "--cpu foobar,+x2apic,+x2apicagain,-distest,forbid=foo,forbid=bar,disable=distest2,optional=opttest,require=reqtest,match=strict,vendor=meee",
+        # Simple --numatune
+        "--numatune 1,2,3,5-7,^6",
       ],
 
       "invalid" : [
@@ -432,6 +467,8 @@ args_dict = {
         "--vcpus foo=bar",
         # --cpu host, but no host CPU in caps
         "--cpu host",
+        # Non-escaped numatune
+        "--numatune 1-3,4,mode=strict",
       ],
 
     }, # category 'cpuram'
@@ -474,6 +511,10 @@ args_dict = {
         "--hvm --nodisks --pxe --security label=foobar.label,a1,z2,b3,type=static",
         # --security static with commas 2
         "--hvm --nodisks --pxe --security label=foobar.label,a1,z2,b3",
+        # --filesystem simple
+        "--hvm --pxe --filesystem /foo/source,/bar/target",
+        # --filesystem template
+        "--hvm --pxe --filesystem template_name,/,type=template",
       ],
 
       "invalid": [
@@ -504,9 +545,17 @@ args_dict = {
         ("--hvm --cdrom %(EXISTIMG2)s --file %(EXISTIMG1)s --os-variant win2k3 --wait 0 --vcpus cores=4", "w2k3-cdrom"),
         # Lot's of devices
         ("--hvm --pxe "
-         "--disk %(EXISTIMG1)s,cache=writeback,io=threads,perms=sh "
-         "--disk %(NEWIMG1)s,sparse=false,size=.001,perms=ro "
-         "--serial tcp,host=:2222,mode=bind,protocol=telnet ",
+         "--disk %(EXISTIMG1)s,cache=writeback,io=threads,perms=sh,serial=WD-WMAP9A966149 "
+         "--disk %(NEWIMG1)s,sparse=false,size=.001,perms=ro,error_policy=enospace "
+         "--disk device=cdrom "
+         "--serial tcp,host=:2222,mode=bind,protocol=telnet "
+         "--filesystem /source,/target,mode=squash "
+         "--network user,mac=12:34:56:78:11:22 "
+         "--network bridge=foobar,model=virtio "
+         "--channel spicevmc "
+         "--smartcard passthrough,type=spicevmc "
+         "--security type=static,label='system_u:object_r:svirt_image_t:s0:c100,c200' "
+         """ --numatune \\"1-3,5\\",mode=preferred """,
          "many-devices"),
       ],
 
@@ -542,8 +591,6 @@ args_dict = {
         "--network=FOO",
         # Invalid mac
         "--network=network:default --mac 1234",
-        # More mac addrs than nics
-        "--network user --mac 00:11:22:33:44:EF --mac 00:11:22:33:44:AB",
         # Mixing bridge and network
         "--network user --bridge foo0",
         # Colliding macaddr
@@ -673,8 +720,23 @@ args_dict = {
   ],
 
 },
+
+"lxc" : {
+  "args": "--connect %(LXCURI)s --noautoconsole --name foolxc --ram 64",
+
+  "valid" : [],
+  "invalid" : [],
+
+  "compare" : [
+    ("", "default"),
+    ("--filesystem /source,/", "fs-default"),
+    ("--init /usr/bin/httpd", "manual-init"),
+  ],
+
+}, # lxc
+
     "prompt" : [ " --connect %(TESTURI)s --debug --prompt" ]
-  },
+  }, # virt-install
 
 
 
@@ -706,6 +768,8 @@ args_dict = {
         #"--original-xml %(CLONE_STORAGE_XML)s --file /cross-pool/clonevol",
         # XML w/ non-existent storage, with --preserve
         "--original-xml %(CLONE_NOEXIST_XML)s --file %(EXISTIMG1)s --preserve",
+        # Overwriting existing VM
+        "-o test -n test-many-devices --replace",
       ],
 
       "invalid": [
@@ -786,10 +850,10 @@ args_dict = {
 
 
   'virt-image': {
-    "globalargs" : " --connect %(TESTURI)s -d %(IMAGE_XML)s",
+    "globalargs" : " --connect %(TESTURI)s --debug",
 
     "general" : {
-      "args" : "--name test-image",
+      "args" : "--name test-image %(IMAGE_XML)s",
 
       "valid": [
         # All default values
@@ -813,7 +877,7 @@ args_dict = {
      }, # category 'general'
 
     "graphics" : {
-      "args" : "--name test-image --boot 0",
+      "args" : "--name test-image --boot 0 %(IMAGE_XML)s",
 
       "valid": [
         # SDL
@@ -830,25 +894,30 @@ args_dict = {
 
       "valid" : [
         # Colliding VM name w/ --replace
-        "--name test --replace",
+        "--name test --replace %(IMAGE_XML)s",
       ],
       "invalid" : [
         # No name specified, and no prompt flag
-        "",
+        "%(IMAGE_XML)s",
         # Colliding VM name without --replace
-        "--name test",
+        "--name test %(IMAGE_XML)s",
       ],
 
       "compare" : [
-        ("--name foobar --ram 64 --os-variant winxp --boot 0", "image-boot0"),
-        ("--name foobar --ram 64 --network user,model=e1000 --boot 1",
+        ("--name foobar --ram 64 --os-variant winxp --boot 0 %(IMAGE_XML)s",
+         "image-boot0"),
+        ("--name foobar --ram 64 --network user,model=e1000 --boot 1 "
+         "%(IMAGE_XML)s",
          "image-boot1"),
+        ("--name foobar --ram 64 --boot 0 "
+         "%(IMAGE_NOGFX_XML)s",
+         "image-nogfx"),
       ]
 
      }, # category 'misc'
 
      "network": {
-      "args": "--name test-image --boot 0 --nographics",
+      "args": "--name test-image --boot 0 --nographics %(IMAGE_XML)s",
 
       "valid": [
         # user networking
