@@ -74,7 +74,7 @@ ACCESS__CONN="Access to a protected member _conn"
 
 # There isn't a clean API way to access this functions from the API, but
 # they provide info that is needed. These need need to be fixed.
-PROT_MEM_BUGS="protected member (_lookup_osdict_key|_OS_TYPES|_prepare_install|_create_devices|_cleanup_install|_install_bootconfig|_channels|_get_caps|_open_test_uri)|'virtinst.FullVirtGuest' has no '_OS_TYPES'"
+PROT_MEM_BUGS="protected member (_lookup_osdict_key|_OS_TYPES|_prepare_install|_create_devices|_cleanup_install|_install_bootconfig|_channels|_get_caps|_open_test_uri|_set_rhel6)|'virtinst.FullVirtGuest' has no '_OS_TYPES'"
 
 
 # This doesn't belong here, but makes sure I don't break the test suite :)
