@@ -14,6 +14,15 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
+import unittest
+import time
+import logging
+import re
+import platform
+import sys
+
+import utils
+import virtinst
 import virtinst.OSDistro as OSDistro
 from virtinst.OSDistro import FedoraDistro
 from virtinst.OSDistro import SuseDistro
@@ -23,15 +32,8 @@ from virtinst.OSDistro import SLDistro
 from virtinst.OSDistro import UbuntuDistro
 from virtinst.OSDistro import MandrivaDistro
 
-import unittest
-import time
-import logging
-import re
-import urlgrabber.progress
-import platform
-import utils
 import libvirt
-import virtinst
+import urlgrabber.progress
 
 # Filters for including/excluding certain distros.
 MATCH_FILTER = ".*"
@@ -56,9 +58,6 @@ OLDCENTOS_BASEURL = "http://vault.centos.org/%s/os/%s"
 MANDRIVA_BASEURL = "http://ftp.uwsg.indiana.edu/linux/mandrake/official/%s/%s/"
 SCIENTIFIC_BASEURL = "http://ftp.scientificlinux.org/linux/scientific/%s/%s/"
 
-# Regex matching distro names that don't have xen kernels.
-NOXEN_FILTER = ".*ubuntu.*|.*etch.*|.*mandriva.*|.*lenny-64.*|.*centos-4.0.*|.*scientific-4.0.*"
-
 # Doesn't appear to be a simple boot iso in newer suse trees
 NOBOOTISO_FILTER = ".*opensuse11.*|.*opensuse10.3.*|.*opensuse10.0.*"
 
@@ -120,12 +119,18 @@ urls = {
     },
 
     # Debian Distros
-    "debian-lenny-32" : {
-        'i386' : DEBIAN_BASEURL % ("lenny", "i386"),
+    "debian-lenny-64" : {
+        "noxen": True,
+        'x86_64': DEBIAN_BASEURL % ("lenny", "amd64"),
         'distro': ("linux", None)
     },
-    "debian-lenny-64" : {
-        'x86_64': DEBIAN_BASEURL % ("lenny", "amd64"),
+    "debian-squeeze" : {
+        'i386' : DEBIAN_BASEURL % ("squeeze", "i386"),
+        'x86_64': DEBIAN_BASEURL % ("squeeze", "amd64"),
+        'distro': ("linux", None)
+    },
+    "debian-wheezy" : {
+        'x86_64': DEBIAN_BASEURL % ("wheezy", "amd64"),
         'distro': ("linux", None)
     },
     "debian-daily" : {
@@ -149,6 +154,7 @@ urls = {
         'distro': ("linux", None)
     },
     "centos-4.0" : {
+        "noxen": True,
         'x86_64' : OLDCENTOS_BASEURL % ("4.0", "x86_64"),
         'distro': ("linux", None)
     },
@@ -171,16 +177,28 @@ urls = {
         'distro': ("linux", None)
     },
     "scientific-4.0" : {
+        "noxen": True,
         'x86_64': SCIENTIFIC_BASEURL % ("40", "x86_64"),
         'distro': ("linux", None)
     },
 
     # Ubuntu
     "ubuntu-hardy" : {
+        "noxen": True,
         'i386': UBUNTU_BASEURL % ("hardy", "i386"),
         'x86_64': UBUNTU_BASEURL % ("hardy", "amd64"),
         'distro': ("linux", None)
     },
+    "ubuntu-lucid" : {
+        'i386': UBUNTU_BASEURL % ("lucid", "i386"),
+        'x86_64': UBUNTU_BASEURL % ("lucid", "amd64"),
+        'distro': ("linux", None)
+    },
+    "ubuntu-maverick" : {
+        'i386': UBUNTU_BASEURL % ("maverick", "i386"),
+        'x86_64': UBUNTU_BASEURL % ("maverick", "amd64"),
+        'distro': ("linux", None)
+    },
     "ubuntu-natty" : {
         'i386': UBUNTU_BASEURL % ("natty", "i386"),
         'x86_64': UBUNTU_BASEURL % ("natty", "amd64"),
@@ -188,25 +206,23 @@ urls = {
     },
 
     # Mandriva
-    "mandriva-2007.1" : {
-        'x86_64': MANDRIVA_BASEURL % ("2007.1", "x86_64"),
+    "mandriva-2009.1" : {
+        "noxen": True,
+        'i386': MANDRIVA_BASEURL % ("2009.1", "i586"),
+        'x86_64': MANDRIVA_BASEURL % ("2009.1", "x86_64"),
         'distro': ("linux", None)
     },
-    "mandriva-2008.1" : {
-        'x86_64': MANDRIVA_BASEURL % ("2008.1", "x86_64"),
+    "mandriva-2010.2" : {
+        "noxen": True,
+        'i386': MANDRIVA_BASEURL % ("2010.2", "i586"),
+        'x86_64': MANDRIVA_BASEURL % ("2010.2", "x86_64"),
         'distro': ("linux", None)
     },
-    "mandriva-2009.0" : {
-        'i386': MANDRIVA_BASEURL % ("2009.0", "i586"),
-        'x86_64': MANDRIVA_BASEURL % ("2009.0", "x86_64"),
-        'distro': ("linux", None)
-    },
-
 }
 
 
 testconn = libvirt.open("test:///default")
-testguest = virtinst.FullVirtGuest(connection=testconn, installer=virtinst.DistroInstaller())
+testguest = virtinst.FullVirtGuest(conn=testconn, installer=virtinst.DistroInstaller())
 
 class TestURLFetch(unittest.TestCase):
 
@@ -214,7 +230,7 @@ class TestURLFetch(unittest.TestCase):
     def setUp(self):
         self.meter = urlgrabber.progress.BaseMeter()
         if utils.get_debug():
-            self.meter = urlgrabber.progress.TextMeter()
+            self.meter = urlgrabber.progress.TextMeter(fo=sys.stdout)
 
     def _fetchLocalMedia(self, mediapath):
         arch = platform.machine()
@@ -231,12 +247,11 @@ class TestURLFetch(unittest.TestCase):
             fetcher.cleanupLocation()
 
 
-    def _fetchFromURLDict(self, distname, url, arch, distro_info):
+    def _fetchFromURLDict(self, distname, url, arch, distro_info, check_xen):
         logging.debug("\nDistro='%s' arch='%s' url=%s" % \
                       (distname, arch, url))
 
         fetcher = OSDistro._fetcherForURI(url, "/tmp")
-
         try:
             fetcher.prepareLocation()
         except Exception, e:
@@ -247,7 +262,8 @@ class TestURLFetch(unittest.TestCase):
             return
 
         try:
-            self._grabURLMedia(fetcher, distname, url, arch, distro_info)
+            self._grabURLMedia(fetcher, distname, url, arch, distro_info,
+                               check_xen)
         finally:
             fetcher.cleanupLocation()
 
@@ -274,11 +290,8 @@ class TestURLFetch(unittest.TestCase):
             if v:
                 testguest.os_variant = v
 
-    def _grabURLMedia(self, fetcher, distname, url, arch, distro_info=None):
-
-        check_xen = True
-        if re.match(r"%s" % NOXEN_FILTER, distname):
-            check_xen = False
+    def _grabURLMedia(self, fetcher, distname, url, arch, distro_info,
+                      check_xen):
 
         hvmstore = self._getStore(fetcher, url, "hvm", arch)
 
@@ -387,19 +400,24 @@ class TestURLFetch(unittest.TestCase):
                 logging.debug("Excluding '%s' from exclude filter." % label)
                 continue
 
+            check_xen = not bool(urls[label].get("noxen"))
             if "distro" in urls[label]:
                 distro_info = urls[label]["distro"]
 
             for arch, url in urls[label].items():
-                if arch == "distro":
+                if arch == "distro" or arch == "noxen":
                     continue
 
                 try:
                     print "Testing %s-%s : %s" % (label, arch, url)
-                    self._fetchFromURLDict(label, url, arch, distro_info)
+                    self._fetchFromURLDict(label, url, arch, distro_info,
+                                           check_xen)
                 except AssertionError:
                     print "%s-%s FAILED." % (label, arch)
                     assertions += 1
+                except Exception:
+                    print "%s-%s ERROR." % (label, arch)
+                    assertions += 1
 
         if assertions != 0:
             raise AssertionError("Found %d errors in URL suite." % assertions)
