@@ -34,7 +34,7 @@ class XMLParseTest(unittest.TestCase):
 
     def _roundtrip_compare(self, filename):
         expectXML = sanitize_file_xml(file(filename).read())
-        guest = virtinst.Guest(connection=conn, parsexml=expectXML)
+        guest = virtinst.Guest(conn=conn, parsexml=expectXML)
         actualXML = guest.get_config_xml()
         utils.diff_compare(actualXML, expect_out=expectXML)
 
@@ -86,7 +86,7 @@ class XMLParseTest(unittest.TestCase):
         """
         infile  = "tests/xmlparse-xml/change-guest-in.xml"
         outfile = "tests/xmlparse-xml/change-guest-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         check = self._make_checker(guest)
@@ -95,7 +95,7 @@ class XMLParseTest(unittest.TestCase):
         check("description", None, "Hey desc changed")
         check("maxvcpus", 5, 30)
         check("vcpus", 30, 22)
-        check("cpuset", "1-3", "1-5,15")
+        check("cpuset", "1-3", "1-8,^6", "1-5,15")
         check("maxmemory", 400, 500)
         check("memory", 200, 1000)
         check("maxmemory", 1000, 2000)
@@ -117,6 +117,8 @@ class XMLParseTest(unittest.TestCase):
         check("os_type", "hvm", "xen")
         check("arch", "i686", None)
         check("machine", "foobar", "pc-0.11")
+        check("loader", None, "/foo/loader")
+        check("init", None, "/sbin/init")
 
         check = self._make_checker(guest.installer.bootconfig)
         check("bootorder", ["hd"], ["fd"])
@@ -153,12 +155,16 @@ class XMLParseTest(unittest.TestCase):
         guest.cpu.remove_feature(guest.cpu.features[1])
         guest.cpu.add_feature("addfeature")
 
+        check = self._make_checker(guest.numatune)
+        check("memory_mode", "interleave", "strict", None)
+        check("memory_nodeset", "1-5,^3,7", "2,4,6")
+
         self._alter_compare(guest.get_config_xml(), outfile)
 
     def testAlterMinimalGuest(self):
         infile  = "tests/xmlparse-xml/change-minimal-guest-in.xml"
         outfile = "tests/xmlparse-xml/change-minimal-guest-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         check = self._make_checker(guest.features)
@@ -192,7 +198,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterBootMulti(self):
         infile  = "tests/xmlparse-xml/change-boot-multi-in.xml"
         outfile = "tests/xmlparse-xml/change-boot-multi-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         check = self._make_checker(guest.installer.bootconfig)
@@ -207,7 +213,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterBootKernel(self):
         infile  = "tests/xmlparse-xml/change-boot-kernel-in.xml"
         outfile = "tests/xmlparse-xml/change-boot-kernel-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         check = self._make_checker(guest.installer.bootconfig)
@@ -226,7 +232,7 @@ class XMLParseTest(unittest.TestCase):
         """
         infile  = "tests/xmlparse-xml/change-disk-in.xml"
         outfile = "tests/xmlparse-xml/change-disk-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         # XXX: Set size up front. VirtualDisk validation is kind of
@@ -243,6 +249,7 @@ class XMLParseTest(unittest.TestCase):
         check("path", "/tmp/test.img", "/dev/loop0")
         check("driver_name", None, "test")
         check("driver_type", None, "foobar")
+        check("serial", "WD-WMAP9A966149", "frob")
 
         check = self._make_checker(disk2)
         check("path", "/dev/loop0", None)
@@ -250,6 +257,7 @@ class XMLParseTest(unittest.TestCase):
         check("read_only", True, False)
         check("target", None, "fde")
         check("bus", None, "fdc")
+        check("error_policy", "stop", None)
 
         check = self._make_checker(disk3)
         check("path", None, "/default-pool/default-vol")
@@ -270,7 +278,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterChars(self):
         infile  = "tests/xmlparse-xml/change-chars-in.xml"
         outfile = "tests/xmlparse-xml/change-chars-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         serial1     = guest.get_devices("serial")[0]
@@ -329,7 +337,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterControllers(self):
         infile  = "tests/xmlparse-xml/change-controllers-in.xml"
         outfile = "tests/xmlparse-xml/change-controllers-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("controller")[0]
@@ -355,7 +363,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterNics(self):
         infile  = "tests/xmlparse-xml/change-nics-in.xml"
         outfile = "tests/xmlparse-xml/change-nics-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("interface")[0]
@@ -397,6 +405,7 @@ class XMLParseTest(unittest.TestCase):
         check = self._make_checker(dev5)
         check("type", "direct")
         check("source_dev", "eth0.1")
+        check("source_mode", "vepa", "bridge")
 
         virtualport = dev5.virtualport
         check = self._make_checker(virtualport)
@@ -412,7 +421,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterInputs(self):
         infile  = "tests/xmlparse-xml/change-inputs-in.xml"
         outfile = "tests/xmlparse-xml/change-inputs-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("input")[0]
@@ -432,7 +441,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterGraphics(self):
         infile  = "tests/xmlparse-xml/change-graphics-in.xml"
         outfile = "tests/xmlparse-xml/change-graphics-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("graphics")[0]
@@ -476,7 +485,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterVideos(self):
         infile  = "tests/xmlparse-xml/change-videos-in.xml"
         outfile = "tests/xmlparse-xml/change-videos-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("video")[0]
@@ -501,7 +510,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterHostdevs(self):
         infile  = "tests/xmlparse-xml/change-hostdevs-in.xml"
         outfile = "tests/xmlparse-xml/change-hostdevs-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("hostdev")[0]
@@ -538,7 +547,7 @@ class XMLParseTest(unittest.TestCase):
     def testAlterWatchdogs(self):
         infile  = "tests/xmlparse-xml/change-watchdogs-in.xml"
         outfile = "tests/xmlparse-xml/change-watchdogs-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("watchdog")[0]
@@ -546,13 +555,41 @@ class XMLParseTest(unittest.TestCase):
         check("model", "ib700", "i6300esb")
         check("action", "none", "poweroff")
 
+        self._alter_compare(guest.get_config_xml(), outfile)
+
+    def testAlterFilesystems(self):
+        devtype = "filesystem"
+        infile  = "tests/xmlparse-xml/change-%ss-in.xml" % devtype
+        outfile = "tests/xmlparse-xml/change-%ss-out.xml" % devtype
+        guest = virtinst.Guest(conn=conn,
+                               parsexml=file(infile).read())
+
+        dev1 = guest.get_devices(devtype)[0]
+        dev2 = guest.get_devices(devtype)[1]
+        dev3 = guest.get_devices(devtype)[2]
+
         check = self._make_checker(dev1)
+        check("type", None, "mount")
+        check("mode", None, "passthrough")
+        check("source", "/foo/bar", "/new/path")
+        check("target", "/bar/baz", "/new/target")
+
+        check = self._make_checker(dev2)
+        check("type", "template")
+        check("mode", None, "mapped")
+        check("source", "template_fedora", "template_new")
+        check("target", "/bar/baz")
+
+        check = self._make_checker(dev3)
+        check("type", "mount", None)
+        check("mode", "squash", None)
+
         self._alter_compare(guest.get_config_xml(), outfile)
 
     def testAlterSounds(self):
         infile  = "tests/xmlparse-xml/change-sounds-in.xml"
         outfile = "tests/xmlparse-xml/change-sounds-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("sound")[0]
@@ -570,10 +607,66 @@ class XMLParseTest(unittest.TestCase):
 
         self._alter_compare(guest.get_config_xml(), outfile)
 
+    def testAlterAddr(self):
+        infile  = "tests/xmlparse-xml/change-addr-in.xml"
+        outfile = "tests/xmlparse-xml/change-addr-out.xml"
+        guest = virtinst.Guest(conn=conn,
+                               parsexml=file(infile).read())
+
+        dev1 = guest.get_devices("disk")[0]
+        dev2 = guest.get_devices("controller")[0]
+        dev3 = guest.get_devices("channel")[0]
+
+        check = self._make_checker(dev1.address)
+        check("type", "drive", "pci")
+        check("type", "pci", "drive")
+        check("controller", "3", "1")
+        check("bus", "5", "4")
+        check("unit", "33", "32")
+        check = self._make_checker(dev1.alias)
+        check("name", "foo2", None)
+
+        check = self._make_checker(dev2.address)
+        check("type", "pci")
+        check("domain", "0x0000", "0x0001")
+        check("bus", "0x00", "4")
+        check("slot", "0x04", "10")
+        check("function", "0x7", "0x6")
+        check = self._make_checker(dev2.alias)
+        check("name", None, "frob")
+
+        check = self._make_checker(dev3.address)
+        check("type", "virtio-serial")
+        check("controller", "0")
+        check("bus", "0")
+        check("port", "2", "4")
+        check = self._make_checker(dev3.alias)
+        check("name", "channel0", "channel1")
+
+        self._alter_compare(guest.get_config_xml(), outfile)
+
+    def testAlterSmartCard(self):
+        infile  = "tests/xmlparse-xml/change-smartcard-in.xml"
+        outfile = "tests/xmlparse-xml/change-smartcard-out.xml"
+        guest = virtinst.Guest(conn=conn,
+                               parsexml=file(infile).read())
+
+        dev1 = guest.get_devices("smartcard")[0]
+        dev2 = guest.get_devices("smartcard")[1]
+
+        check = self._make_checker(dev1)
+        check("type", None, "tcp")
+
+        check = self._make_checker(dev2)
+        check("mode", "passthrough", "host")
+        check("type", "spicevmc", None)
+
+        self._alter_compare(guest.get_config_xml(), outfile)
+
     def testConsoleCompat(self):
         infile  = "tests/xmlparse-xml/console-compat-in.xml"
         outfile = "tests/xmlparse-xml/console-compat-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         dev1 = guest.get_devices("console")[0]
@@ -585,7 +678,7 @@ class XMLParseTest(unittest.TestCase):
     def testAddRemoveDevices(self):
         infile  = "tests/xmlparse-xml/add-devices-in.xml"
         outfile = "tests/xmlparse-xml/add-devices-out.xml"
-        guest = virtinst.Guest(connection=conn,
+        guest = virtinst.Guest(conn=conn,
                                parsexml=file(infile).read())
 
         rmdev = guest.get_devices("disk")[2]
@@ -605,7 +698,7 @@ class XMLParseTest(unittest.TestCase):
     def testChangeKVMMedia(self):
         infile  = "tests/xmlparse-xml/change-media-in.xml"
         outfile = "tests/xmlparse-xml/change-media-out.xml"
-        guest = virtinst.Guest(connection=kvmconn,
+        guest = virtinst.Guest(conn=kvmconn,
                                parsexml=file(infile).read())
 
         disk = guest.get_devices("disk")[0]
