@@ -278,7 +278,8 @@ class ovf_parser(formats.parser):
 
         res = False
         try:
-            res = bool(get_xml_path(xml, "/ovf:Envelope"))
+            if xml.count("</Envelope>"):
+                res = bool(get_xml_path(xml, "/ovf:Envelope"))
         except Exception, e:
             logging.debug("Error parsing OVF XML: %s" % str(e))
 
@@ -294,6 +295,7 @@ class ovf_parser(formats.parser):
         infile = open(input_file, "r")
         xml = infile.read()
         infile.close()
+        logging.debug("Importing OVF XML:\n%s" % xml)
 
         return _xml_wrapper(xml, ovf_parser._import_file)
 
@@ -444,16 +446,4 @@ class ovf_parser(formats.parser):
         """
         raise NotImplementedError
 
-    @staticmethod
-    def export_file(vm, output_file):
-        """
-        Export a configuration file.
-        @vm vm configuration instance
-        @output_file Output file
-
-        Raises ValueError if configuration is not suitable, or another
-        exception on failure to write the output file.
-        """
-        raise NotImplementedError
-
 formats.register_parser(ovf_parser)
