@@ -44,7 +44,7 @@ import Guest
 from VirtualNetworkInterface import VirtualNetworkInterface
 from VirtualDisk import VirtualDisk
 from virtinst import Storage
-from virtinst import _virtinst as _
+from virtinst import _gettext as _
 import _util
 
 def _listify(val):
@@ -115,11 +115,12 @@ class CloneDesign(object):
     CLONE_POLICY_NO_SHAREABLE  = 2
     CLONE_POLICY_NO_EMPTYMEDIA = 3
 
-    def __init__(self, connection):
-        # hypervisor connection
-        if not isinstance(connection, libvirt.virConnect):
+    def __init__(self, connection=None, conn=None):
+        conn = conn or connection
+
+        if not isinstance(conn, libvirt.virConnect):
             raise ValueError(_("Connection must be a 'virConnect' instance."))
-        self._hyper_conn = connection
+        self._hyper_conn = conn
 
         # original guest name or uuid
         self._original_guest        = None
@@ -149,7 +150,7 @@ class CloneDesign(object):
                                     self.CLONE_POLICY_NO_EMPTYMEDIA]
 
         # Throwaway guest to use for easy validation
-        self._valid_guest        = Guest.Guest(connection=connection)
+        self._valid_guest        = Guest.Guest(conn=conn)
 
         # Generate a random UUID at the start
         while 1:
@@ -189,14 +190,6 @@ class CloneDesign(object):
         except ValueError, e:
             raise ValueError(_("Invalid name for new guest: %s") % e)
 
-        # Make sure new VM name isn't taken.
-        try:
-            if self._hyper_conn.lookupByName(name) is not None:
-                raise ValueError(_("Domain name '%s' already in use.") %
-                                 name)
-        except libvirt.libvirtError:
-            pass
-
         self._clone_name = name
     clone_name = property(get_clone_name, set_clone_name,
                           doc="Name to use for the new guest clone.")
@@ -390,6 +383,13 @@ class CloneDesign(object):
                                  "domain state is not checked before "
                                  "cloning.")
 
+    def _get_replace(self):
+        return self._valid_guest.replace
+    def _set_replace(self, val):
+        self._valid_guest.replace = bool(val)
+    replace = property(_get_replace, _set_replace,
+                       doc="f enabled, don't check for clone name collision, "
+                           "simply undefine any conflicting guest.")
     # Functional methods
 
     def setup_original(self):
@@ -526,6 +526,8 @@ class CloneDesign(object):
         self.setup_clone()
         logging.debug("Clone guest xml is\n%s" % (self._clone_xml))
 
+    def remove_original_vm(self, force=None):
+        return self._valid_guest.remove_original_vm(force=force)
 
     # Private helper functions
 
@@ -672,8 +674,10 @@ def start_duplicate(design, meter=None):
 
     dom = None
     try:
-        # Define domain first so we can catch any xml errors before duplicating
-        # storage
+        # Replace orig VM if required
+        design.remove_original_vm()
+
+        # Define domain early to catch any xml errors before duping storage
         dom = design.original_conn.defineXML(design.clone_xml)
 
         if design.preserve == True:
