@@ -55,12 +55,13 @@ from _util import xml_escape as escape
 
 import _util
 import support
-from virtinst import _virtinst as _
+from virtinst import _gettext as _
 
 DEFAULT_DEV_TARGET = "/dev"
 DEFAULT_LVM_TARGET_BASE = "/dev/"
 DEFAULT_DIR_TARGET_BASE = "/var/lib/libvirt/images/"
 DEFAULT_SCSI_TARGET = "/dev/disk/by-path"
+DEFAULT_MPATH_TARGET = "/dev/mapper"
 
 # Pulled from libvirt, used for building on older versions
 VIR_STORAGE_VOL_FILE = 0
@@ -240,7 +241,8 @@ class StoragePool(StorageObject):
     TYPE_LOGICAL = "logical"
     TYPE_DISK    = "disk"
     TYPE_ISCSI   = "iscsi"
-    TYPE_SCSI   = "scsi"
+    TYPE_SCSI    = "scsi"
+    TYPE_MPATH   = "mpath"
     """@group Types: TYPE_*"""
 
     # Pool type descriptions for use in higher level programs
@@ -251,7 +253,8 @@ class StoragePool(StorageObject):
     _types[TYPE_LOGICAL] = _("LVM Volume Group")
     _types[TYPE_DISK]    = _("Physical Disk Device")
     _types[TYPE_ISCSI]   = _("iSCSI Target")
-    _types[TYPE_SCSI]    = _("SCSI host adapter")
+    _types[TYPE_SCSI]    = _("SCSI Host Adapter")
+    _types[TYPE_MPATH]   = _("Multipath Device Enumerator")
 
     def get_pool_class(ptype):
         """
@@ -276,6 +279,8 @@ class StoragePool(StorageObject):
             return iSCSIPool
         if ptype == StoragePool.TYPE_SCSI:
             return SCSIPool
+        if ptype == StoragePool.TYPE_MPATH:
+            return MultipathPool
     get_pool_class = staticmethod(get_pool_class)
 
     def get_volume_for_pool(pool_type):
@@ -436,7 +441,7 @@ class StoragePool(StorageObject):
                "%s" % src_xml + \
                "%s" % tar_xml
 
-    def install(self, meter=None, create=False, build=False):
+    def install(self, meter=None, create=False, build=False, autostart=False):
         """
         Install storage pool xml.
         """
@@ -468,6 +473,12 @@ class StoragePool(StorageObject):
             except Exception, e:
                 errmsg = _("Could not start storage pool: %s" % str(e))
 
+        if autostart and not errmsg:
+            try:
+                pool.setAutostart(True)
+            except Exception, e:
+                errmsg = _("Could not set pool autostart flag: %s" % str(e))
+
         if errmsg:
             # Try and clean up the leftover pool
             try:
@@ -725,12 +736,12 @@ class LogicalPool(StoragePool):
             xml += "    <name>%s</name>\n" % self.source_name
         return xml
 
-    def install(self, meter=None, create=False, build=False):
+    def install(self, meter=None, create=False, build=False, autostart=False):
         if build and not self.source_path:
             raise ValueError(_("Must explicitly specify source path if "
                                "building pool"))
         return StoragePool.install(self, meter=meter, create=create,
-                                   build=build)
+                                   build=build, autostart=False)
 
 class DiskPool(StoragePool):
     """
@@ -787,12 +798,12 @@ class DiskPool(StoragePool):
         xml += """    <device path="%s"/>\n""" % escape(self.source_path)
         return xml
 
-    def install(self, meter=None, create=False, build=False):
+    def install(self, meter=None, create=False, build=False, autostart=False):
         if self.format == "auto" and build:
             raise ValueError(_("Must explicitly specify disk format if "
                                "formatting disk device."))
         return StoragePool.install(self, meter=meter, create=create,
-                                   build=build)
+                                   build=build, autostart=autostart)
 
 class iSCSIPool(StoragePool):
     """
@@ -807,8 +818,7 @@ class iSCSIPool(StoragePool):
                                  " volumes."))
 
     def get_volume_class():
-        raise NotImplementedError(_("iSCSI volume creation is not "
-                                    "implemented."))
+        raise NotImplementedError(_("iSCSI volume creation is not supported."))
     get_volume_class = staticmethod(get_volume_class)
 
     def __init__(self, conn, name, source_path=None, host=None,
@@ -821,6 +831,8 @@ class iSCSIPool(StoragePool):
         if host:
             self.host = host
 
+        self._iqn = None
+
     # Need to overwrite pool *_source_path since iscsi device isn't
     # a fully qualified path
     def get_source_path(self):
@@ -830,6 +842,13 @@ class iSCSIPool(StoragePool):
     source_path = property(get_source_path, set_source_path,
                            doc=_("Path on the host that is being shared."))
 
+    def _get_iqn(self):
+        return self._iqn
+    def _set_iqn(self, val):
+        self._iqn = val
+    iqn = property(_get_iqn, _set_iqn,
+                        doc=_("iSCSI initiator qualified name"))
+
     def _get_default_target_path(self):
         return DEFAULT_SCSI_TARGET
 
@@ -842,8 +861,17 @@ class iSCSIPool(StoragePool):
             raise RuntimeError(_("Hostname is required"))
         if not self.source_path:
             raise RuntimeError(_("Host path is required"))
-        xml = """    <host name="%s"/>\n""" % self.host + \
-              """    <device path="%s"/>\n""" % escape(self.source_path)
+
+        iqn_xml = ""
+        if self.iqn:
+            iqn_xml += """    <initiator>\n"""
+            iqn_xml += """      <iqn name="%s"/>\n""" % escape(self.iqn)
+            iqn_xml += """    </initiator>\n"""
+
+        xml  = """    <host name="%s"/>\n""" % self.host
+        xml += """    <device path="%s"/>\n""" % escape(self.source_path)
+        xml += iqn_xml
+
         return xml
 
 class SCSIPool(StoragePool):
@@ -857,8 +885,7 @@ class SCSIPool(StoragePool):
                                  " volumes."))
 
     def get_volume_class():
-        raise NotImplementedError(_("SCSI volume creation is not "
-                                    "implemented."))
+        raise NotImplementedError(_("SCSI volume creation is not supported."))
     get_volume_class = staticmethod(get_volume_class)
 
     def __init__(self, conn, name, source_path=None,
@@ -891,7 +918,34 @@ class SCSIPool(StoragePool):
         xml = """    <adapter name="%s"/>\n""" % escape(self.source_path)
         return xml
 
+class MultipathPool(StoragePool):
+    """
+    Create a Multipath based storage pool
+    """
 
+    target_path = property(StoragePool.get_target_path,
+                           StoragePool.set_target_path,
+                           doc=_("Root location for identifying new storage"
+                                 " volumes."))
+
+    def get_volume_class():
+        raise NotImplementedError(_("Multipath volume creation is not "
+                                    "supported."))
+    get_volume_class = staticmethod(get_volume_class)
+
+    def __init__(self, conn, name, target_path=None, uuid=None):
+        StoragePool.__init__(self, name=name, type=StoragePool.TYPE_MPATH,
+                             uuid=uuid, target_path=target_path, conn=conn)
+
+    def _get_default_target_path(self):
+        return DEFAULT_MPATH_TARGET
+
+    def _get_target_xml(self):
+        xml = "    <path>%s</path>\n" % escape(self.target_path)
+        return xml
+
+    def _get_source_xml(self):
+        return ""
 
 """
 Storage Volume classes
@@ -954,7 +1008,7 @@ class StorageVolume(StorageObject):
     get_volume_for_pool = staticmethod(get_volume_for_pool)
 
     def find_free_name(name, pool_object=None, pool_name=None, conn=None,
-                       suffix=""):
+                       suffix="", collidelist=None):
         """
         Finds a name similar (or equal) to passed 'name' that is not in use
         by another pool
@@ -968,16 +1022,19 @@ class StorageVolume(StorageObject):
 
         Ex name="test", suffix=".img" -> name-3.img
 
+        @collidelist: An extra list of names to check for collision
         @returns: A free name
         @rtype: C{str}
         """
-
-        pool_object = StorageVolume.lookup_pool_by_name(pool_object=pool_object,
-                                                        pool_name=pool_name,
-                                                        conn=conn)
+        collidelist = collidelist or []
+        pool_object = StorageVolume.lookup_pool_by_name(
+                                                    pool_object=pool_object,
+                                                    pool_name=pool_name,
+                                                    conn=conn)
         pool_object.refresh(0)
+
         return _util.generate_name(name, pool_object.storageVolLookupByName,
-                                   suffix)
+                                   suffix, collidelist=collidelist)
     find_free_name = staticmethod(find_free_name)
 
     def lookup_pool_by_name(pool_object=None, pool_name=None, conn=None):
