@@ -23,8 +23,7 @@ import logging
 import libvirt
 
 from virtinst import support
-from virtinst import _util
-from virtinst import _virtinst as _
+from virtinst import _gettext as _
 from XMLBuilderDomain import _xml_property
 
 class VirtualHostDevice(VirtualDevice.VirtualDevice):
@@ -99,7 +98,7 @@ class VirtualHostDevice(VirtualDevice.VirtualDevice):
             return
 
         self.managed = True
-        if _util.get_uri_driver(self.conn.getURI()).lower() == "xen":
+        if self.is_xen():
             self.managed = False
 
 
